@@ -8,6 +8,10 @@
 #  This should be placed in a directory, "tests", at the root of your project. It assumes that ../$MY_PACKAGE_MODULE is the path to your test module, and will create a symlink to it in order to run tests.
 #  The tests should be found in $MY_TEST_DIRECTORY in given "tests" folder.
 
+
+# NOTE: Since version 1.2.3, you can also import this (like from a graphical application) and call the "main()" function.
+#  All of the following globals are the defaults,  but can be overridden when calling main() (params have the same name as the globals).
+
 import imp
 import os
 
@@ -15,6 +19,7 @@ import glob
 import subprocess
 import sys
 
+# URL to current version of GoodTests.py - You only need to change this if you host an internal copy.
 GOODTESTS_URL = 'https://raw.githubusercontent.com/kata198/GoodTests/master/GoodTests.py'
 
 # This should be your module name, and can be any relative or absolute path, or just a module name. 
@@ -23,12 +28,23 @@ MY_PACKAGE_MODULE = 'AdvancedHTMLParser'
 
 #  Normally, you want to test the codebase during development, so you don't care about the site-packages installed version.
 #     If you want to allow testing with any module by @MY_PACKAGE_MODULE in the python path, change this to True.
-ALLOW_SITE_INSTALL = True
+ALLOW_SITE_INSTALL = False
 
 # This is the test directory that should contain all your tests. This should be a directory in your "tests" folder
 MY_TEST_DIRECTORY = 'AdvancedHTMLParserTests'
 
+__version__ = '1.2.3'
+__version_tuple__ = (1, 2, 3)
+
 def findGoodTests():
+    '''
+        findGoodTests - Tries to find GoodTests.py
+
+        @return <dict> {
+            'path' <str> -> Path to GoodTests.py (for execution)
+            'success' <bool> -> True/False if we successfully found GoodTests.py
+        }
+    '''
     pathSplit = os.environ['PATH'].split(':')
     if '.' not in pathSplit:
         pathSplit = ['.'] + pathSplit
@@ -50,7 +66,15 @@ def findGoodTests():
         "success" : success 
     }
 
-def download_goodTests():
+def download_goodTests(GOODTESTS_URL=None):
+    '''
+        download_goodTests - Attempts to download GoodTests, using the default global url (or one provided).
+    
+        @return <int> - 0 on success (program should continue), otherwise non-zero (program should abort with this exit status)
+    '''
+    if GOODTESTS_URL is None:
+        GOODTESTS_URL = globals()['GOODTESTS_URL']
+
     validAnswer = False
     while validAnswer == False:
         sys.stdout.write('GoodTests not found. Would you like to install it to local folder? (y/n): ')
@@ -63,7 +87,7 @@ def download_goodTests():
 
     if answer == 'n':
         sys.stderr.write('Cannot run tests without installing GoodTests. http://pypi.python.org/pypi/GoodTests or https://github.com/kata198/Goodtests\n')
-        sys.exit(1)
+        return 1
     try:
         import urllib2 as urllib
     except ImportError:
@@ -76,7 +100,7 @@ def download_goodTests():
             res = pipe.wait()
             if res != 0:
                 sys.stderr.write('Failed to install GoodTests with pip ordirect download. aborting.\n')
-                sys.exit(1)
+                return 1
     try:
         response = urllib.urlopen(GOODTESTS_URL)
         contents = response.read()
@@ -84,13 +108,13 @@ def download_goodTests():
             contents = contents.decode('ascii')
     except Exception as e:
         sys.stderr.write('Failed to download GoodTests.py from "%s"\n%s\n' %(GOODTESTS_URL, str(e)))
-        sys.exit(1)
+        return 1
     try:
         with open('GoodTests.py', 'w') as f:
             f.write(contents)
     except Exception as e:
         sys.stderr.write('Failed to write to GoodTests.py\n%s\n' %(str(e,)))
-        sys.exit(1)
+        return 1
     try:
         os.chmod('GoodTests.py', 0o775)
     except:
@@ -100,12 +124,38 @@ def download_goodTests():
         import GoodTests
     except ImportError:
         sys.stderr.write('Seemed to download GoodTests okay, but still cannot import. Aborting.\n')
-        sys.exit(1)
-    
+        return 1
 
-if __name__ == '__main__':
+    return 0
+
+
+def main(thisDir=None, additionalArgs=[], MY_PACKAGE_MODULE=None, ALLOW_SITE_INSTALL=None, MY_TEST_DIRECTORY=None, GOODTESTS_URL=None):
+    '''
+        Do the work - Try to find GoodTests.py, else prompt to download it, then run the tests.
+
+        @param thisDir <None/str> - None to use default (directory this test file is in, or if not obtainable, current directory).
+        @param additionalArgs <list> - Any additional args to pass to GoodTests.py
+
+        Remainder of params take their global (top of file) defaults unless explicitly set here. See top of file for documentation.
+
+        @return <int> - Exit code of application. 0 on success, non-zero on failure.
+
+            TODO: Standardize return codes so external applications can derive failure without parsing error strings.
+    '''
+
+    if MY_PACKAGE_MODULE is None:
+        MY_PACKAGE_MODULE = globals()['MY_PACKAGE_MODULE']
+    if ALLOW_SITE_INSTALL is None:
+        ALLOW_SITE_INSTALL = globals()['ALLOW_SITE_INSTALL']
+    if MY_TEST_DIRECTORY is None:
+        MY_TEST_DIRECTORY = globals()['MY_TEST_DIRECTORY']
+    if GOODTESTS_URL is None:
+        GOODTESTS_URL = globals()['GOODTESTS_URL']
+   
+
+    if not thisDir:
+        thisDir = os.path.dirname(__file__)
 
-    thisDir = os.path.dirname(__file__)
     if not thisDir:
         thisDir = str(os.getcwd())
     elif not thisDir.startswith('/'):
@@ -121,11 +171,13 @@ if __name__ == '__main__':
     
     goodTestsInfo = findGoodTests()
     if goodTestsInfo['success'] is False:
-        download_goodTests()
+        downloadRet = download_goodTests(GOODTESTS_URL)
+        if downloadRet != 0:
+            return downloadRet
         goodTestsInfo = findGoodTests()
         if goodTestsInfo['success'] is False:
             sys.stderr.write('Could not download or find GoodTests.py. Try to download it yourself using "pip install GoodTests", or wget %s\n' %( GOODTESTS_URL,))
-            sys.exit(1)
+            return 1
 
     PYTHON_ENDINGS = ('.py', '.pyc', '.pyo')
 
@@ -146,24 +198,28 @@ if __name__ == '__main__':
         except ImportError:
             # COMPAT WITH PREVIOUS runTests.py: Try plain module in parent directory
             foundIt = False
+            oldSysPath = sys.path[:]
+            sys.path = [os.path.realpath(os.getcwd() + os.sep + '..' + os.sep)]
             try:
-                imp.find_module('..' + os.sep + MY_PACKAGE_MODULE)
+                imp.find_module(MY_PACKAGE_MODULE)
                 foundIt = True
-            except ImportError:
+                sys.path = oldSysPath
+            except ImportError as e:
+                sys.path = oldSysPath
                 if not ALLOW_SITE_INSTALL:
                     sys.stderr.write('Cannot find "%s" locally.\n' %(MY_PACKAGE_MODULE,))
-                    sys.exit(2)
+                    return 2
                 else:
                     try:
                         __import__(baseName)
                     except:
                         sys.stderr.write('Cannot find "%s" locally or in global python path.\n' %(MY_PACKAGE_MODULE,))
-                        sys.exit(2)
+                        return 2
 
             if foundIt is True:
-                newPath = os.getcwd() + os.sep + '..' + os.sep
+                newPath = os.path.realpath(os.getcwd() + os.sep + '..' + os.sep)
         if inCurrentDir is True:
-            newPath = os.getcwd() + os.sep + '..' + os.sep
+            newPath = os.path.realpath(os.getcwd() + os.sep + '..' + os.sep)
     
     if newPath:
         newPythonPath = [newPath] + [x for x in os.environ.get('PYTHONPATH', '').split(':') if x]
@@ -178,19 +234,26 @@ if __name__ == '__main__':
 
         if e.name != MY_PACKAGE_MODULE:
             sys.stderr.write('Error while importing %s: %s\n Likely this is another dependency that needs to be installed\nPerhaps run "pip install %s" or install the providing package.\n\n' %(e.name, str(e), e.name))
-            sys.exit(1)
+            return 1
         sys.stderr.write('Could not import %s. Either install it or otherwise add to PYTHONPATH\n%s\n' %(MY_PACKAGE_MODULE, str(e)))
-        sys.exit(1)
+        return 1
 
     if not os.path.isdir(MY_TEST_DIRECTORY):
         if not os.path.exists(MY_TEST_DIRECTORY):
             sys.stderr.write('Cannot find test directory: %s\n' %(MY_TEST_DIRECTORY,))
         else:
             sys.stderr.write('Provided test directory, "%s" is not a directory.\n' %(MY_TEST_DIRECTORY,))
-        sys.exit(3)
+        return 3
 
     sys.stdout.write('Starting test..\n')
     sys.stdout.flush()
     sys.stderr.flush()
-    pipe = subprocess.Popen([goodTestsInfo['path']] + sys.argv[1:] + [MY_TEST_DIRECTORY], env=os.environ, shell=False)
+    pipe = subprocess.Popen([goodTestsInfo['path']] + additionalArgs + [MY_TEST_DIRECTORY], env=os.environ, shell=False)
     pipe.wait()
+
+    return 0
+
+
+if __name__ == '__main__':
+    ret = main(None, sys.argv[1:])
+    sys.exit(ret)
